#include "defs.h"
#include "kernel-main.h"
#include "../networking/communicate.h"


/*
 * This is part of JEliza 2.3.
 * Copyright 2006 by Tobias Schulz
 * WWW: http://jeliza.ch.to/
 *
 * JEliza is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * JEliza is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU GPL
 * along with JEliza (file gpl.txt) ; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifndef linux
#include <windows.h>
#endif


auto_ptr<jeliza::string> jelizathread1_logs;
auto_ptr<jeliza::string> jelizathread1_talking;

bool server_exit = true;
bool jeliza::fast_exit = false;
void init_it();

int main(int argc, char* argv[]) {
#ifdef JELIZA_DEBUG
#ifndef linux
    if (AllocConsole()) {
        freopen("CON", "w", stdout);
        freopen("CON", "w", stderr);
    }
#endif
#endif

    vector<string> args;
    for (int x = 1; x < argc; ++x) {
        args.push_back(string(argv[x]));
    }

    for (unsigned int x = 0; x < args.size(); ++x) {
        if (args[x] == "-noexit") {
            server_exit = false;
        }
    }


    jeliza::jeliza_manager* jm = new jeliza::jeliza_manager;
    jm->make_current();

    jeliza::comm_init_server();

    Py_SetProgramName(argv[0]);
    PyEval_InitThreads();

    asio::thread t1(init_it);

    speech_init();

    t1.join();

    return 0;
}


void jeliza::log(std::string str) {
    cout << str << endl << flush;
    jeliza::comm_send(jeliza::string("LOG:" + str));
}

int AKTUELLE_PROZENT = 0;

void jeliza::update_prozent(float fl) {
    int i = fl / 2;
    i *= 2;
    if (AKTUELLE_PROZENT != i) {
        AKTUELLE_PROZENT = i;

        stringstream sst;
        sst << i;
        std::string s;
        sst >> s;

        comm_send("PERCENT:" + s);
    }
}


void ask(std::string str) {
    (*(jeliza::jeliza_manager::get_current())) << str;
}

void get_db_string(jeliza::strings parts) {
    jeliza::string db("HERE_IS_DB_STRING:");
    db += jeliza::string(jeliza::jeliza_manager::get_current()->get().get_db_manager().get_string()).replace(":", "::");
    db += ":";
    db += parts[1];
    db.set(db.replace(string("\r"), string("")).replace(string("\n"), string("<BR>")));

    jeliza::comm_send(db.ref());
}

void get_online_db_string(jeliza::strings parts) {
    string db = "HERE_IS_ONLINE_DB_STRING:";

    for (unsigned int x = 1; x < parts.size(); x++) {
        if (parts[x].size() > 0) {
            db += jeliza::download(parts[x].get_strip().ref());
        }
    }

    ofstream o1(("lang_" + jeliza::get_lang_str() + "/jeliza-standard.xml").ref().c_str());
    o1 << db << endl;
    o1.close();
    
    ofstream o2(("lang_" + jeliza::get_lang_str() + "/to_do.xml").ref().c_str());
    o2.close();

    db = jeliza::string(db).replace(string("\r"), string("")).replace(string("\n"), string("<BR>")).ref();

    jeliza::comm_send(db);
}

void import_db(jeliza::strings parts) {
    jeliza::global_fertig = false;

    ofstream o("temp/import_db.tmp");
    o << jeliza::string(parts[1]).replace_nocase(jeliza::string(string("<BR>")), jeliza::string(string("\n"))).ref();
    o.close();

    jeliza::db_manager db_man;
    db_man.new_from("temp/import_db.tmp");

    for (jeliza::DB::iterator it = db_man.get().begin(); it != db_man.get().end(); it++) {
        jeliza::jeliza_manager::get_current()->get().get_db_manager().push(*it);
    }

    jeliza::jeliza_manager::get_current()->get().get_db_manager().save(("lang_" + jeliza::get_lang_str() + "/jeliza-standard.xml").ref());

    jeliza::comm_send("IMPORT_DB_SUCCESS:.");
}

void jeliza::comm_new(jeliza::string s) {
    s = s.replace("::", "----");
    s = s.replace(";", ",");
    s = s.replace(":", ";");

    jeliza::strings parts;
    util::split(s, ";", parts);

    for (unsigned int x = 0; x < parts.size(); x++) {
        parts[x] = parts[x].replace("----", ":");
    }


    if (parts.size() >= 2) {
        if (parts[0] == "QUESTION") {
            asio::thread t(boost::bind(ask, parts[1].ref()));
        }
        if (parts[0] == "EXIT") {
            if (server_exit) {
            	jeliza::next_genus = "4";
            	jeliza::next_word_type = "9";
            	jeliza::msleep(500);
                jeliza::abbauen();
            	
//                jeliza::jeliza_manager::destroy_current();
                std::exit(0);
            }
        }
        if (parts[0] == "GET_DB_STRING") {
            asio::thread t(boost::bind(get_db_string, parts));
        }
        if (parts[0] == "GET_ONLINE_DB_STRING") {
            asio::thread t(boost::bind(get_online_db_string, parts));
        }
        if (parts[0] == "IMPORT_DB") {
            asio::thread t(boost::bind(import_db, parts));
        }
        if (parts[0] == "INIT") {
            jeliza::jeliza_manager::get_current()->get().init_databases();

            ofstream o("temp/new_sentence.tmp");
            o << "!!init";
            o.close();
        }
        if (parts[0] == "WWW_SURF_MODE") {
            if (parts[1].contains("1")) {
                www_surf_mode = true;
            }
            else if (parts[1].contains("0")) {
                www_surf_mode = false;
            }
        }
        if (parts[0] == "OFFLINE_MODE") {
            if (parts[1].contains("1")) {
                offline_mode = true;
            }
            else if (parts[1].contains("0")) {
                offline_mode = false;
            }
        }
        if (parts[0] == "CLEAR") {
            if (parts[1] == "TALKING") {
                ofstream o("temp/clear_talking.tmp");
                o << "clear" << endl;
                o.close();
            }
        }
        if (parts[0] == "USE_ONLINE_DB") {
            if (parts[1] == "YES") {
                jeliza::use_online_db = true;
            }
            else {
                jeliza::use_online_db = false;
            }
        }
        if (parts[0] == "PROXY") {
            cout << "Setting Proxy: " << parts[1].ref() << endl;
            log("Setting Proxy: " + parts[1].ref());
            jeliza::proxy = parts[1].ref();
            stringstream sst;
            sst << parts[2].ref();
            unsigned int prox_p;
            sst >> prox_p;
            jeliza::proxy_port = prox_p;
        }
        if (parts[0] == "HERE_IS_WORD_TYPE") {
            jeliza::next_word_type = parts[1].ref();
        }
        if (parts[0] == "HERE_IS_GENUS") {
            jeliza::next_genus = parts[1].ref();
        }
    }
}

void speech_init() {
}



void speak(string str) {
    comm_send("SPEAK:" + jeliza::string(str).replace("\n", ". ").replace_nocase_safe(s(":"), s("::")));
}

void jeliza::display_sentence(jeliza::string s) {
    comm_send("DISPLAY:" + s.replace("\n", "<br>"));

    jeliza::update_prozent(0);
}

void www_search() {
    long count_loop = 0;

    while (jeliza::global_fertig) {
        {
            if (count_loop > 2 && !offline_mode && www_surf_mode && jeliza::global_fertig) {
                jeliza::strings ans;

                if (wikipedia_words.size() > 0) {
                    string word = wikipedia_words[0].ref();
                    jeliza::strings wikipedia_words2;

                    ans << NULL << search_in_wikipedia_acticle(jeliza::string(word));

                    for (unsigned int g = 1; g < wikipedia_words.size(); g++) {
                        wikipedia_words2.push_back(wikipedia_words[g]);
                    }
                    wikipedia_words << NULL << wikipedia_words2;
                }

                for (unsigned int g = 0; g < ans.size(); g++) {
                    clogger << "- Neuer Satz: " << ans[g].ref() << endl;
                }

                if (ans.size() > 0) {
                    (*jeliza::jeliza_manager::get_current()) << ans;
                }
            }
        }
        if (count_loop >= 0) {
            count_loop++;
        }
        if (jeliza::global_fertig) {
            asio::deadline_timer timer(io_service, boost::posix_time::seconds(60));
            timer.wait();
        }
    }
}

void init_it() {
    jeliza::jeliza_manager::get_current()->start();

    jeliza::update_prozent(0);

    www_search();
}

